﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Shapes;

namespace AffineTransformDemo
{
    public partial class Page : UserControl
    {
        Dot draggedDot;
        Point startPointMouse;
        Point startPointDot;

        public Page()
        {
            InitializeComponent();
        }

        protected override void OnMouseLeftButtonDown(MouseButtonEventArgs args)
        {
            base.OnMouseLeftButtonDown(args);

            if (args.OriginalSource is Path)
            {
                DependencyObject parent = (args.OriginalSource as Path).Parent;

                if (parent is Dot)
                {
                    draggedDot = parent as Dot;
                    startPointDot = draggedDot.Center;
                    startPointMouse = args.GetPosition(this);
                    CaptureMouse();
                }
            }
        }

        protected override void OnMouseMove(MouseEventArgs args)
        {
            base.OnMouseMove(args);

            if (draggedDot != null)
            {
                Point pointMouse = args.GetPosition(this);
                draggedDot.Center = new Point(startPointDot.X - startPointMouse.X + pointMouse.X,
                                              startPointDot.Y - startPointMouse.Y + pointMouse.Y);

                SetNewTransform();
            }
        }

        protected override void OnMouseLeftButtonUp(MouseButtonEventArgs args)
        {
            base.OnMouseLeftButtonUp(args);

            if (draggedDot != null)
            {
                ReleaseMouseCapture();
                draggedDot = null;
            }
        }

        void SetNewTransform()
        {
            double offsetX = dotUL.Center.X;
            double offsetY = dotUL.Center.Y;
            double m11 = (dotUR.Center.X - offsetX) / 192;
            double m12 = (dotUR.Center.Y - offsetY) / 192;
            double m21 = (dotLL.Center.X - offsetX) / 192;
            double m22 = (dotLL.Center.Y - offsetY) / 192;

            xform.Matrix = new Matrix(m11, m12, m21, m22, offsetX, offsetY);

            textM11.Text = m11.ToString("F3");
            textM12.Text = m12.ToString("F3");
            textM21.Text = m21.ToString("F3");
            textM22.Text = m22.ToString("F3");
            textOffsetX.Text = offsetX.ToString("F3");
            textOffsetY.Text = offsetY.ToString("F3");
        }
    }
}
